import face_recognition
import numpy as np
def face_recognition_logic(encodeListKnown, encodeFace, classNames):
    matches = face_recognition.compare_faces(encodeListKnown, encodeFace)
    faceDis = face_recognition.face_distance(encodeListKnown, encodeFace)
    matchIndex = np.argmin(faceDis)

    if matches[matchIndex]:
        name = classNames[matchIndex].upper()
        return matches, matchIndex, name
    else:
        return matches, matchIndex, None


import cv2
import face_recognition
import os
def find_encodings(images):
    encodeList = []
    for img in images:
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        try:
            encode = face_recognition.face_encodings(img)[0]
            encodeList.append(encode)
        except IndexError:
            print("Warning: Face not detected in one of the images. Skipping.")
    return encodeList

def load_images(path):
    images = []
    classNames = []
    
    if not os.path.exists(path):
        print(f"Error: Path '{path}' does not exist.")
        exit()

    myList = os.listdir(path)
    print("Images found:", myList)
    for cl in myList:
        curImg = cv2.imread(f'{path}/{cl}')
        if curImg is not None:
            images.append(curImg)
            classNames.append(os.path.splitext(cl)[0])
        else:
            print(f"Warning: Unable to read image '{cl}'. Skipping.")
    
    return images, classNames


# utils/face_detection.py
import face_recognition

def detect_faces(imgS):
    facesCurFrame = face_recognition.face_locations(imgS)
    encodesCurFrame = face_recognition.face_encodings(imgS, facesCurFrame)
    return facesCurFrame, encodesCurFrame



# utils/attendance.py
import os
from datetime import datetime

activePersons = {}

def markAttendance(name, status):
    today = datetime.now().strftime('%Y-%m-%d')
    now = datetime.now()
    timeNow = now.strftime('%H:%M:%S')

    if not os.path.exists('Attendance.csv'):
        print("Creating Attendance.csv")
        with open('Attendance.csv', 'w') as f:
            f.write('Name,Date,Entry Time,Exit Time\n')

    if status == "enter":
        if name not in activePersons:
            activePersons[name] = now
            print(f"{name} entered at {timeNow}")

    elif status == "exit":
        if name in activePersons:
            entryTime = activePersons.pop(name).strftime('%H:%M:%S')
            print(f"{name} exited at {timeNow} (Entry: {entryTime})")
            with open('Attendance.csv', 'a') as f:
                f.write(f"{name},{today},{entryTime},{timeNow}\n")



# main.py

from .image_processing import load_images, find_encodings
from .attendance import markAttendance
from .face_detection import detect_faces
import cv2
import numpy as np
from .face_recognition_logic import face_recognition_logic

def main():
    path = './student_images'
    images, classNames = load_images(path)
    encodeListKnown = find_encodings(images)
    print('Encoding Complete. Total Encoded Faces:', len(encodeListKnown))

    # Initialize webcam
    cap = cv2.VideoCapture(0)
    if not cap.isOpened():
        print("Error: Could not open webcam.")
        return

    prev_detected = set()  # Track previously detected people in the frame

    while True:
        success, img = cap.read()
        if not success:
            print("Error: Failed to capture image from webcam.")
            break

        # Resize and process the frame for faster performance
        imgS = cv2.resize(img, (0, 0), None, 0.25, 0.25)
        imgS = cv2.cvtColor(imgS, cv2.COLOR_BGR2RGB)

        facesCurFrame, encodesCurFrame = detect_faces(imgS)

        current_detected = set()  # Track currently detected people in the frame

        for encodeFace, faceLoc in zip(encodesCurFrame, facesCurFrame):
            matches, matchIndex, name = face_recognition_logic(encodeListKnown, encodeFace, classNames)

            if matches[matchIndex]:
                current_detected.add(name)

                # If the person is newly detected, mark as "enter"
                if name not in prev_detected:
                    markAttendance(name, "enter")

                # Draw bounding box and name
                y1, x2, y2, x1 = faceLoc
                y1, x2, y2, x1 = y1 * 4, x2 * 4, y2 * 4, x1 * 4
                cv2.rectangle(img, (x1, y1), (x2, y2), (0, 255, 0), 2)
                cv2.rectangle(img, (x1, y2 - 35), (x2, y2), (0, 255, 0), cv2.FILLED)
                cv2.putText(img, name, (x1 + 6, y2 - 6), cv2.FONT_HERSHEY_COMPLEX, 1, (255, 255, 255), 2)

        # Detect people who exited
        for name in prev_detected - current_detected:
            markAttendance(name, "exit")

        prev_detected = current_detected

        # Show the result
        cv2.imshow('Webcam', img)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            print("Exiting program.")
            break

    # Release resources
    cap.release()
    cv2.destroyAllWindows()

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(
    name="face_recognition_atd",  # Package name
    version="2.0.0",  # Version number
    packages=find_packages(),  # Automatically find packages
    install_requires=[  # Dependencies
        "numpy",
        "opencv-python",
        "face-recognition"
    ],
    entry_points={  # Allow running the main script as a command
        'console_scripts': [
            'face_recognition=face_recognition_package.main:main',
        ],
    },
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author="Joshik Raj",
    author_email="joshikrajjojo@gmail.com",
    description="A face recognition package with attendance tracking",
    url="https://github.com/JoshikRaj/face_recognition",
    
)
